"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();
/* eslint-disable no-param-reassign,array-callback-return,consistent-return, max-len */


exports.checkIsFormOK = checkIsFormOK;
exports.rulesRunner = rulesRunner;

var _is_js = require("is_js");

var _is_js2 = _interopRequireDefault(_is_js);

var _collectValuesUtils = require("./collectValuesUtils");

var _helpers = require("./helpers");

var _matchers = require("../ruleHandlers/matchers");

var _matchers2 = _interopRequireDefault(_matchers);

var _initializationUtils = require("./initializationUtils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * If all fields in the state has their status !== error
 * Then we will set the isFormOK to true then return the state.
 * Just mutate the value since it's already a new state object
 *
 */
function checkIsFormOK(schema, componentState) {
  var properties = Object.keys(schema);
  var isError = false;
  properties.some(function (prop) {
    if (prop === 'collectValues') return false;

    if (_is_js2.default.propertyDefined(schema[prop], 'isRequired') && schema[prop].isRequired === false && componentState[prop].status !== _helpers.FieldStatus.error) return false;

    if (componentState[prop].status === _helpers.FieldStatus.error) {
      isError = true;
      return true;
    }

    if (componentState[prop].status === _helpers.FieldStatus.normal) {
      if (_is_js2.default.not.propertyDefined(schema[prop], 'default')) {
        isError = true;
        return true;
      }

      if (schema[prop].default !== componentState[prop].value) {
        isError = true;
        return true;
      }
    }
    return false;
  });
  componentState.isFormOK = !isError;

  return componentState;
}

function handleBeforeValidation(fieldValue, handler) {
  if (_is_js2.default.function(handler)) {
    return handler(fieldValue);
  }
  return fieldValue;
}

function extractUserDefinedMsg(handlerName, schema) {
  var result = { schema: schema, userErrorText: '' };

  // No user message, just return
  if (_is_js2.default.not.array(schema[handlerName])) return result;

  var currentSchema = schema[handlerName];

  // Handle the case where the value of rule is array
  if (_matchers.RuleWhichNeedsArray.includes(handlerName)) {
    // No user message, just return
    if (_is_js2.default.not.array(currentSchema[0])) return result;
  }

  // The most common case: [0] is rule and [1] is errText

  var _currentSchema = _slicedToArray(currentSchema, 2),
      rule = _currentSchema[0],
      errText = _currentSchema[1];

  result.schema[handlerName] = rule;
  result.userErrorText = errText;
  return result;
}

function ruleRunner(ruleName, ruleHandler, fieldName, value, fieldRules) {
  var _extractUserDefinedMs = extractUserDefinedMsg(ruleName, fieldRules),
      schema = _extractUserDefinedMs.schema,
      userErrorText = _extractUserDefinedMs.userErrorText;

  if (_matchers.RuleWhichNeedsBoolean.includes(ruleName)) {
    if (schema[ruleName] === false) return;
  }

  var result = ruleHandler(fieldName, value, schema);
  if (result.isValid) return;

  (0, _helpers.throwError)(value, userErrorText || result.errorText);
}

function grabValueForReliesField(allSchema, allState, reliedFieldName) {
  var result = void 0;

  if (_is_js2.default.propertyDefined(allState, reliedFieldName) && _is_js2.default.propertyDefined(allState[reliedFieldName], "value")) {
    result = allState[reliedFieldName].value;
  } else if (_is_js2.default.propertyDefined(allSchema, "collectValues") && _is_js2.default.propertyDefined(allSchema.collectValues, reliedFieldName)) {
    result = (0, _collectValuesUtils.getNestedValue)(allSchema.collectValues[reliedFieldName], allState);
  }

  return result;
}

function handleReliesOn(fieldReliesOnSchema, fieldState, allSchema, allState) {
  var originalFieldState = _extends({}, fieldState);
  Object.keys(fieldReliesOnSchema).forEach(function (reliedFieldName) {
    var reliesKeySchema = fieldReliesOnSchema[reliedFieldName];
    Object.keys(reliesKeySchema).forEach(function (rule) {

      if (_is_js2.default.not.propertyDefined(_matchers2.default, rule)) return;

      var reliedFieldValue = grabValueForReliesField(allSchema, allState, reliedFieldName);

      try {
        ruleRunner(rule, _matchers2.default[rule], reliedFieldName, reliedFieldValue, // Here we need to swap the field value to the target value
        reliesKeySchema);
      } catch (err) {
        // Restore the original value
        err.value = originalFieldState.value;
        throw err;
      }
    });
  });
}

function handleOnlyWhen(fieldOnlyWhenSchema, fieldState, allSchema, allState) {
  return Object.keys(fieldOnlyWhenSchema).every(function (reliedFieldName) {
    var reliesKeySchema = fieldOnlyWhenSchema[reliedFieldName];

    return Object.keys(reliesKeySchema).every(function (rule) {
      if (_is_js2.default.not.propertyDefined(_matchers2.default, rule)) return;

      var reliedFieldValue = grabValueForReliesField(allSchema, allState, reliedFieldName);

      try {
        ruleRunner(rule, _matchers2.default[rule], reliedFieldName, reliedFieldValue, // Here we need to swap the field value to the target value
        reliesKeySchema);
      } catch (err) {
        return false;
      }
      return true;
    });
  });
}

/**
 * It will run through the user's settings for a field,
 * and try matching to the matchers.js,
 * if according rule could be found,
 * it will then execute the according rule function.
 * For instance:
 * if user sets a `minLength` for a field,
 * This function will invoke the minLength()
 *
 */
function runMatchers(matcher, fieldState, fieldSchema, allSchema, allState) {
  var fieldName = Object.keys(fieldSchema)[0];
  var fieldRules = fieldSchema[fieldName];

  if ('onlyWhen' in fieldRules && _is_js2.default.not.empty(fieldRules.onlyWhen)) {
    var fieldOnlyWhenOnSchema = fieldSchema[fieldName].onlyWhen;
    if (allSchema && allState && fieldOnlyWhenOnSchema) {
      var result = handleOnlyWhen(fieldOnlyWhenOnSchema, _extends({}, fieldState), allSchema, allState);

      if (result === false) {
        fieldState.status = _helpers.FieldStatus.normal;
        return fieldState;
      }
    }
  }

  if ('beforeValidation' in fieldRules && fieldRules.beforeValidation != null && _is_js2.default.function(fieldRules.beforeValidation)) {
    fieldState.value = handleBeforeValidation(fieldState.value, fieldRules.beforeValidation);
  }

  Object.keys(fieldRules).forEach(function (ruleInSchema) {
    if (ruleInSchema === 'reliesOn') {
      var fieldReliesOnSchema = fieldSchema[fieldName].reliesOn;
      if (allSchema && allState && fieldReliesOnSchema) {
        handleReliesOn(fieldReliesOnSchema, fieldState, allSchema, allState);
      }
    } else if (_is_js2.default.propertyDefined(matcher, ruleInSchema)) {
      // eslint-disable-next-line no-use-before-define
      ruleRunner(ruleInSchema, matcher[ruleInSchema], fieldName, fieldState.value, fieldRules);
    }

    // TODO: Do something when the rule is not match
    // else if (ruleInSchema !== 'default') {
    // }
  });
  return fieldState;
}

/**
 * This is the main entry for all validator.
 * It will generate the initial state to start with
 *
 */
function rulesRunner(value, fieldSchema, allSchema, allState) {
  var fieldState = (0, _initializationUtils.createNewFieldState)();
  fieldState.value = value;

  if (_is_js2.default.existy(value) && _is_js2.default.not.empty(value)) {
    fieldState.status = _helpers.FieldStatus.ok;
  }

  return runMatchers(_matchers2.default, fieldState, fieldSchema, allSchema, allState);
}
//# sourceMappingURL=validationUtils.js.map